==> azure-toolkit-common-lib-0.17.0-sources.jar: com/microsoft/azure/toolkit/lib/common/action/Action.java <==
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 */

package com.microsoft.azure.toolkit.lib.common.action;

import com.microsoft.azure.toolkit.lib.common.bundle.AzureString;
import com.microsoft.azure.toolkit.lib.common.entity.IAzureBaseResource;
import com.microsoft.azure.toolkit.lib.common.operation.AzureOperation;
import com.microsoft.azure.toolkit.lib.common.operation.IAzureOperation;
import com.microsoft.azure.toolkit.lib.common.task.AzureTask;
import com.microsoft.azure.toolkit.lib.common.task.AzureTaskManager;
import com.microsoft.azure.toolkit.lib.common.telemetry.AzureTelemetry;
import com.microsoft.azure.toolkit.lib.common.view.IView;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.apache.commons.lang3.StringUtils;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.BiConsumer;
import java.util.function.BiPredicate;
import java.util.function.Consumer;
import java.util.function.Predicate;

@Accessors(chain = true)
public class Action<D> {
    public static final String SOURCE = "ACTION_SOURCE";
    public static final String RESOURCE_TYPE = "resourceType";
    public static final Id<Runnable> REQUIRE_AUTH = Id.of("action.common.requireAuth");
    public static final Id<Void> AUTHENTICATE = Id.of("action.account.authenticate");
    @Nonnull
    private List<AbstractMap.SimpleEntry<BiPredicate<D, ?>, BiConsumer<D, ?>>> handlers = new ArrayList<>();
    @Nullable
    @Getter
    private ActionView.Builder view;
    @Setter
    @Getter
    private boolean authRequired = true;
    /**
     * shortcuts for this action.
     * 1. directly bound to this action if it's IDE-specific type of shortcuts (e.g. {@code ShortcutSet} in IntelliJ).
     * 2. interpreted into native shortcuts first and then bound to this action if it's {@code String[]/String} (e.g. {@code "alt X"}).
     * 3. copy shortcuts from actions specified by this action id and then bound to this action if it's {@link Id} of another action.
     */
    @Setter
    @Getter
    private Object shortcuts;

    public Action(@Nullable ActionView.Builder view) {
        this.view = view;
    }

    public Action(@Nonnull Consumer<D> handler) {
        this.registerHandler((d, e) -> true, (d, e) -> handler.accept(d));
    }

    public <E> Action(@Nonnull BiConsumer<D, E> handler) {
        this.registerHandler((d, e) -> true, handler);
    }

    public Action(@Nonnull Consumer<D> handler, @Nullable ActionView.Builder view) {
        this.view = view;
        this.registerHandler((d, e) -> true, (d, e) -> handler.accept(d));
    }

    public <E> Action(@Nonnull BiConsumer<D, E> handler, @Nullable ActionView.Builder view) {
        this.view = view;
        this.registerHandler((d, e) -> true, handler);
    }

    private Action(@Nonnull List<AbstractMap.SimpleEntry<BiPredicate<D, ?>, BiConsumer<D, ?>>> handlers, @Nullable ActionView.Builder view) {
        this.view = view;
        this.handlers = handlers;
    }

    @Nullable
    public IView.Label getView(D source) {
        return Objects.nonNull(this.view) ? this.view.toActionView(source) : null;
    }

    @SuppressWarnings("unchecked")
    public BiConsumer<D, Object> getHandler(D source, Object e) {
        for (int i = this.handlers.size() - 1; i >= 0; i--) {
            final AbstractMap.SimpleEntry<BiPredicate<D, ?>, BiConsumer<D, ?>> p = this.handlers.get(i);
            final BiPredicate<D, Object> condition = (BiPredicate<D, Object>) p.getKey();
            final BiConsumer<D, Object> handler = (BiConsumer<D, Object>) p.getValue();
            if (condition.test(source, e)) {
                return handler;
            }
        }
        return null;
    }

    public void handle(D source, Object e) {
        final Runnable runnable = () -> {
            final BiConsumer<D, Object> handler = this.getHandler(source, e);
            if (Objects.nonNull(handler)) {
                final AzureString title = Optional.ofNullable(this.view).map(b -> b.title).map(t -> t.apply(source))
                    .orElse(AzureString.fromString(IAzureOperation.UNKNOWN_NAME));
                final AzureTask<Void> task = new AzureTask<>(title, () -> handle(source, e, handler));
                task.setType(AzureOperation.Type.ACTION.name());
                AzureTaskManager.getInstance().runInBackground(task);
            }
        };
        if (this.authRequired) {
            final Action<Runnable> requireAuth = AzureActionManager.getInstance().getAction(REQUIRE_AUTH);
            if (Objects.nonNull(requireAuth)) {
                requireAuth.handle(runnable, e);
            }
        } else {
            runnable.run();
        }
    }

    protected void handle(D source, Object e, BiConsumer<D, Object> handler) {
        if (source instanceof IAzureBaseResource) {
            AzureTelemetry.getActionContext().setProperty("subscriptionId", ((IAzureBaseResource<?, ?>) source).subscriptionId());
            AzureTelemetry.getActionContext().setProperty(RESOURCE_TYPE, source.getClass().getSimpleName());
        }
        handler.accept(source, e);
    }

    public void handle(D source) {
        this.handle(source, null);
    }

    public void registerHandler(@Nonnull Predicate<D> condition, @Nonnull Consumer<D> handler) {
        this.handlers.add(new AbstractMap.SimpleEntry<>((d, e) -> condition.test(d), (d, e) -> handler.accept(d)));
    }

    public <E> void registerHandler(@Nonnull BiPredicate<D, E> condition, @Nonnull BiConsumer<D, E> handler) {
        this.handlers.add(new AbstractMap.SimpleEntry<>(condition, handler));
    }

    public static class Id<D> {
        @Nonnull
        private final String id;

        private Id(@Nonnull String id) {
            this.id = id;
        }

        public static <D> Id<D> of(@Nonnull String id) {
            assert StringUtils.isNotBlank(id) : "action id can not be blank";
            return new Id<>(id);
        }

        @Nonnull
        public String getId() {
            return id;
        }
    }

    public static Action<Void> retryFromFailure(@Nonnull Runnable handler) {
        return new Action<>((v) -> handler.run(), new ActionView.Builder("Retry"));
    }
}

==> azure-toolkit-common-lib-0.19.0-sources.jar: com/microsoft/azure/toolkit/lib/common/action/Action.java <==
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 */

package com.microsoft.azure.toolkit.lib.common.action;

import com.microsoft.azure.toolkit.lib.common.bundle.AzureString;
import com.microsoft.azure.toolkit.lib.common.model.AzResource;
import com.microsoft.azure.toolkit.lib.common.operation.AzureOperation;
import com.microsoft.azure.toolkit.lib.common.operation.IAzureOperation;
import com.microsoft.azure.toolkit.lib.common.task.AzureTask;
import com.microsoft.azure.toolkit.lib.common.task.AzureTaskManager;
import com.microsoft.azure.toolkit.lib.common.telemetry.AzureTelemetry;
import com.microsoft.azure.toolkit.lib.common.view.IView;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.apache.commons.lang3.StringUtils;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.BiConsumer;
import java.util.function.BiPredicate;
import java.util.function.Consumer;
import java.util.function.Predicate;

@Accessors(chain = true)
public class Action<D> {
    public static final String SOURCE = "ACTION_SOURCE";
    public static final String RESOURCE_TYPE = "resourceType";
    public static final Id<Runnable> REQUIRE_AUTH = Id.of("action.common.requireAuth");
    public static final Id<Void> AUTHENTICATE = Id.of("action.account.authenticate");
    @Nonnull
    private List<AbstractMap.SimpleEntry<BiPredicate<D, ?>, BiConsumer<D, ?>>> handlers = new ArrayList<>();
    @Nullable
    @Getter
    private ActionView.Builder view;
    @Setter
    @Getter
    private boolean authRequired = true;
    /**
     * shortcuts for this action.
     * 1. directly bound to this action if it's IDE-specific type of shortcuts (e.g. {@code ShortcutSet} in IntelliJ).
     * 2. interpreted into native shortcuts first and then bound to this action if it's {@code String[]/String} (e.g. {@code "alt X"}).
     * 3. copy shortcuts from actions specified by this action id and then bound to this action if it's {@link Id} of another action.
     */
    @Setter
    @Getter
    private Object shortcuts;

    public Action(@Nullable ActionView.Builder view) {
        this.view = view;
    }

    public Action(@Nonnull Consumer<D> handler) {
        this.registerHandler((d, e) -> true, (d, e) -> handler.accept(d));
    }

    public <E> Action(@Nonnull BiConsumer<D, E> handler) {
        this.registerHandler((d, e) -> true, handler);
    }

    public Action(@Nonnull Consumer<D> handler, @Nullable ActionView.Builder view) {
        this.view = view;
        this.registerHandler((d, e) -> true, (d, e) -> handler.accept(d));
    }

    public <E> Action(@Nonnull BiConsumer<D, E> handler, @Nullable ActionView.Builder view) {
        this.view = view;
        this.registerHandler((d, e) -> true, handler);
    }

    private Action(@Nonnull List<AbstractMap.SimpleEntry<BiPredicate<D, ?>, BiConsumer<D, ?>>> handlers, @Nullable ActionView.Builder view) {
        this.view = view;
        this.handlers = handlers;
    }

    @Nullable
    public IView.Label getView(D source) {
        return Objects.nonNull(this.view) ? this.view.toActionView(source) : null;
    }

    @SuppressWarnings("unchecked")
    public BiConsumer<D, Object> getHandler(D source, Object e) {
        for (int i = this.handlers.size() - 1; i >= 0; i--) {
            final AbstractMap.SimpleEntry<BiPredicate<D, ?>, BiConsumer<D, ?>> p = this.handlers.get(i);
            final BiPredicate<D, Object> condition = (BiPredicate<D, Object>) p.getKey();
            final BiConsumer<D, Object> handler = (BiConsumer<D, Object>) p.getValue();
            if (condition.test(source, e)) {
                return handler;
            }
        }
        return null;
    }

    public void handle(D source, Object e) {
        final Runnable runnable = () -> {
            final BiConsumer<D, Object> handler = this.getHandler(source, e);
            if (Objects.nonNull(handler)) {
                final AzureString title = Optional.ofNullable(this.view).map(b -> b.title).map(t -> t.apply(source))
                    .orElse(AzureString.fromString(IAzureOperation.UNKNOWN_NAME));
                final AzureTask<Void> task = new AzureTask<>(title, () -> handle(source, e, handler));
                task.setType(AzureOperation.Type.ACTION.name());
                AzureTaskManager.getInstance().runInBackground(task);
            }
        };
        if (this.authRequired) {
            final Action<Runnable> requireAuth = AzureActionManager.getInstance().getAction(REQUIRE_AUTH);
            if (Objects.nonNull(requireAuth)) {
                requireAuth.handle(runnable, e);
            }
        } else {
            runnable.run();
        }
    }

    protected void handle(D source, Object e, BiConsumer<D, Object> handler) {
        if (source instanceof AzResource) {
            Optional.of(AzureTelemetry.getContext()).map(AzureTelemetry.Context::getActionParent).ifPresent(c -> {
                c.setProperty("subscriptionId", ((AzResource<?, ?, ?>) source).getSubscriptionId());
                c.setProperty("resourceType", source.getClass().getSimpleName());
            });
        }
        handler.accept(source, e);
    }

    public void handle(D source) {
        this.handle(source, null);
    }

    public void registerHandler(@Nonnull Predicate<D> condition, @Nonnull Consumer<D> handler) {
        this.handlers.add(new AbstractMap.SimpleEntry<>((d, e) -> condition.test(d), (d, e) -> handler.accept(d)));
    }

    public <E> void registerHandler(@Nonnull BiPredicate<D, E> condition, @Nonnull BiConsumer<D, E> handler) {
        this.handlers.add(new AbstractMap.SimpleEntry<>(condition, handler));
    }

    public static class Id<D> {
        @Nonnull
        private final String id;

        private Id(@Nonnull String id) {
            this.id = id;
        }

        public static <D> Id<D> of(@Nonnull String id) {
            assert StringUtils.isNotBlank(id) : "action id can not be blank";
            return new Id<>(id);
        }

        @Nonnull
        public String getId() {
            return id;
        }
    }

    public static Action<Void> retryFromFailure(@Nonnull Runnable handler) {
        return new Action<>((v) -> handler.run(), new ActionView.Builder("Retry"));
    }
}
